﻿using System.Collections.Generic;
using System.Text.Json.Serialization;

namespace ActFareServiceSample
{
    /// <summary>
    /// 基底レスポンス。
    /// </summary>
    public class BaseResponse
    {
        /// <summary>
        /// エラーコード。
        /// </summary>
        [JsonPropertyName("errorCode")]
        public int ErrorCode { get; set; }
        /// <summary>
        /// エラーメッセージ。
        /// </summary>
        [JsonPropertyName("errorMessage")]
        public string ErrorMessage { get; set; }
    }

    /// <summary>
    /// ユーザー情報。
    /// </summary>
    public class GetInformationResponse : BaseResponse
    {
        /// <summary>
        /// 名称。
        /// </summary>
        [JsonPropertyName("name")]
        public string Name { get; set; }
        /// <summary>
        /// 有効期限。
        /// </summary>
        [JsonPropertyName("expiryDate")]
        public string ExpiryDate { get; set; }
        /// <summary>
        /// 計算用トークン最大数。
        /// </summary>
        [JsonPropertyName("maxToken")]
        public int MaxToken { get; set; }
        /// <summary>
        /// 最大計算回数。
        /// </summary>
        [JsonPropertyName("maxCalcCount")]
        public int MaxCalcCount { get; set; }
        /// <summary>
        /// 計算回数。
        /// </summary>
        [JsonPropertyName("calcCount")]
        public int CalcCount { get; set; }
        /// <summary>
        /// 最大計算レコード数。
        /// </summary>
        [JsonPropertyName("maxCalcRecord")]
        public int MaxCalcRecord { get; set; }
        /// <summary>
        /// 最大運賃表登録数。
        /// </summary>
        [JsonPropertyName("maxTariff")]
        public int MaxTariff { get; set; }
        /// <summary>
        /// 最大距離呈登録数。
        /// </summary>
        [JsonPropertyName("maxDistance")]
        public int MaxDistance { get; set; }
        /// <summary>
        /// 最大運賃プロファイル登録数。
        /// </summary>
        [JsonPropertyName("maxProfile")]
        public int MaxProfile { get; set; }
    }

    /// <summary>
    /// トークン作成レスポンス。
    /// </summary>
    public class CreateTokenResponse : BaseResponse
    {
        /// <summary>
        /// トークン。
        /// </summary>
        [JsonPropertyName("token")]
        public string Token { get; set; }
    }

    /// <summary>
    /// 運賃計算リクエスト。
    /// </summary>
    class CalcRequest
    {
        /// <summary>
        /// プロファイルID。
        /// </summary>
        [JsonPropertyName("id")]
        public string Id { get; set; }
        /// <summary>
        /// 計算基準日。
        /// </summary>
        [JsonPropertyName("date")]
        public string Date { get; set; }
        /// <summary>
        /// 適用距離出力フラグ。
        /// </summary>
        public bool OutputDistances { get; set; }
        /// <summary>
        /// 計算レコードリスト。
        /// </summary>
        [JsonPropertyName("records")]
        public List<object[]> Records { get; set; }
    }

    /// <summary>
    /// 運賃計算レスポンス（運賃のみ）。
    /// </summary>
    class SimpleCalcResponse : BaseResponse
    {
        /// <summary>
        /// 運賃リスト。
        /// </summary>
        [JsonPropertyName("fares")]
        public List<decimal> Fares { get; set; }
    }

    /// <summary>
    /// 運賃計算レスポンス（運賃と適用距離）。
    /// </summary>
    class CalcResponse : BaseResponse
    {
        /// <summary>
        /// 適用距離リスト。
        /// </summary>
        [JsonPropertyName("distances")]
        public List<decimal> Distances { get; set; }
        /// <summary>
        /// 運賃リスト。
        /// </summary>
        [JsonPropertyName("fares")]
        public List<decimal> Fares { get; set; }
    }
}
