﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Json;
using System.Text.Unicode;
using System.Threading.Tasks;
using System.Web;
using System.Windows.Forms;

namespace ActFareServiceSample
{
    public partial class Form1 : Form
    {
        private static readonly HttpClient client = new HttpClient { Timeout = new TimeSpan(0, 5, 0) };

        public Form1()
        {
            InitializeComponent();

            textBoxUrl.Text = ConfigurationManager.AppSettings["Url"];
        }

        private async void ButtonCalcTsumiawase_ClickAsync(object sender, EventArgs e)
        {
            try
            {
                // 前回のデータをクリア
                textBoxLog.Text = "";
                client.DefaultRequestHeaders.Clear();

                // リクエストのヘッダーにX-ACT-Authorizationを設定
                // 「ユーザー名:パスワード」をBASE64エンコードしたものを値として設定
                byte[] bs = Encoding.UTF8.GetBytes(textBoxUserName.Text + ":" + textBoxPassword.Text);
                client.DefaultRequestHeaders.Add("X-ACT-Authorization", Convert.ToBase64String(bs));

                // ユーザー情報（1回で計算できる最大レコード数）を取得
                GetInformationResponse informationResponse = await GetUserInformation();

                // トークンを作成
                CreateTokenResponse createTokenResponse = await CreateToken();

                // リクエストのヘッダーにX-ACT-FARE-Tokenを設定
                // 作成したトークンを値として設定
                client.DefaultRequestHeaders.Add("X-ACT-FARE-Token", createTokenResponse.Token);

                try
                {
                    // 計算に使用する運賃表を登録
                    if (!await AddTariff(Path.Combine(Application.StartupPath, @"Data\Tariff\tsumiawase110.txt")))
                    {
                        return;
                    }

                    // 計算に使用する距離呈を登録（使用しない場合は省略可）
                    if (!await AddDistance(Path.Combine(Application.StartupPath, @"Data\Distance\distance1000.txt")))
                    {
                        return;
                    }

                    // 計算に使用するプロファイルを登録
                    if (!await AddProfile(Path.Combine(Application.StartupPath, @"Data\Profile\tsumiawase110.txt")))
                    {
                        return;
                    }

                    // 運賃を計算
                    if (!await CalcTsumiawase(informationResponse.MaxCalcRecord))
                    {
                        return;
                    }
                }
                finally
                {
                    // トークンを削除
                    BaseResponse tokenResponse = await DeleteToken();
                }
            }
            catch (Exception ex)
            {
                textBoxLog.Text += ex.ToString();
            }
        }

        private async void ButtonCalcParcel_Click(object sender, EventArgs e)
        {
            try
            {
                // 前回のデータをクリア
                textBoxLog.Text = "";
                client.DefaultRequestHeaders.Clear();

                // リクエストのヘッダーにX-ACT-Authorizationを設定
                // 「ユーザー名:パスワード」をBASE64エンコードしたものを値として設定
                byte[] bs = Encoding.UTF8.GetBytes(textBoxUserName.Text + ":" + textBoxPassword.Text);
                client.DefaultRequestHeaders.Add("X-ACT-Authorization", Convert.ToBase64String(bs));

                // ユーザー情報（1回で計算できる最大レコード数）を取得
                GetInformationResponse informationResponse = await GetUserInformation();

                // トークンを作成
                CreateTokenResponse createTokenResponse = await CreateToken();

                // リクエストのヘッダーにX-ACT-FARE-Tokenを設定
                // 作成したトークンを値として設定
                client.DefaultRequestHeaders.Add("X-ACT-FARE-Token", createTokenResponse.Token);

                try
                {
                    // 計算に使用する運賃表を登録
                    if (!await AddTariff(Path.Combine(Application.StartupPath, @"Data\Tariff\parcel141.txt")))
                    {
                        return;
                    }

                    // 計算に使用するプロファイルを登録
                    if (!await AddProfile(Path.Combine(Application.StartupPath, @"Data\Profile\parcel140.txt")))
                    {
                        return;
                    }

                    // 運賃を計算
                    if (!await CalcParcel(informationResponse.MaxCalcRecord))
                    {
                        return;
                    }
                }
                finally
                {
                    // トークンを削除
                    BaseResponse tokenResponse = await DeleteToken();
                }
            }
            catch (Exception ex)
            {
                textBoxLog.Text += ex.ToString();
            }
        }

        private async void ButtonCalcArea_Click(object sender, EventArgs e)
        {
            try
            {
                // 前回のデータをクリア
                textBoxLog.Text = "";
                client.DefaultRequestHeaders.Clear();

                // リクエストのヘッダーにX-ACT-Authorizationを設定
                // 「ユーザー名:パスワード」をBASE64エンコードしたものを値として設定
                byte[] bs = Encoding.UTF8.GetBytes(textBoxUserName.Text + ":" + textBoxPassword.Text);
                client.DefaultRequestHeaders.Add("X-ACT-Authorization", Convert.ToBase64String(bs));

                // ユーザー情報（1回で計算できる最大レコード数）を取得
                GetInformationResponse informationResponse = await GetUserInformation();

                // トークンを作成
                CreateTokenResponse createTokenResponse = await CreateToken();

                // リクエストのヘッダーにX-ACT-FARE-Tokenを設定
                // 作成したトークンを値として設定
                client.DefaultRequestHeaders.Add("X-ACT-FARE-Token", createTokenResponse.Token);

                try
                {
                    // 計算に使用する運賃表を登録
                    if (!await AddTariff(Path.Combine(Application.StartupPath, @"Data\Tariff\area170.txt")))
                    {
                        return;
                    }

                    // 計算に使用するプロファイルを登録
                    if (!await AddProfile(Path.Combine(Application.StartupPath, @"Data\Profile\area170.txt")))
                    {
                        return;
                    }

                    // 運賃を計算
                    if (!await CalcArea(informationResponse.MaxCalcRecord))
                    {
                        return;
                    }
                }
                finally
                {
                    // トークンを削除
                    BaseResponse tokenResponse = await DeleteToken();
                }
            }
            catch (Exception ex)
            {
                textBoxLog.Text += ex.ToString();
            }
        }

        private async void ButtonCalcCode_Click(object sender, EventArgs e)
        {
            try
            {
                // 前回のデータをクリア
                textBoxLog.Text = "";
                client.DefaultRequestHeaders.Clear();

                // リクエストのヘッダーにX-ACT-Authorizationを設定
                // 「ユーザー名:パスワード」をBASE64エンコードしたものを値として設定
                byte[] bs = Encoding.UTF8.GetBytes(textBoxUserName.Text + ":" + textBoxPassword.Text);
                client.DefaultRequestHeaders.Add("X-ACT-Authorization", Convert.ToBase64String(bs));

                // ユーザー情報（1回で計算できる最大レコード数）を取得
                GetInformationResponse informationResponse = await GetUserInformation();

                // トークンを作成
                CreateTokenResponse createTokenResponse = await CreateToken();
                if (createTokenResponse.ErrorCode != 0)
                {
                    return;
                }
                // リクエストのヘッダーにX-ACT-FARE-Tokenを設定
                // 作成したトークンを値として設定
                client.DefaultRequestHeaders.Add("X-ACT-FARE-Token", createTokenResponse.Token);

                try
                {
                    // 計算に使用する運賃表を登録
                    if (!await AddTariff(Path.Combine(Application.StartupPath, @"Data\Tariff\code180.txt")))
                    {
                        return;
                    }

                    // 計算に使用するプロファイルを登録
                    if (!await AddProfile(Path.Combine(Application.StartupPath, @"Data\Profile\code180.txt")))
                    {
                        return;
                    }

                    // 運賃を計算
                    if (!await CalcCode(informationResponse.MaxCalcRecord))
                    {
                        return;
                    }
                }
                finally
                {
                    // トークンを削除
                    BaseResponse deleteTokenResponse = await DeleteToken();
                }
            }
            catch (Exception ex)
            {
                textBoxLog.Text += ex.ToString();
            }
        }

        private async Task<GetInformationResponse> GetUserInformation()
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "information";
            HttpResponseMessage httpResponse = await client.GetAsync(url);
            string responseBody = await httpResponse.Content.ReadAsStringAsync();
            GetInformationResponse response = JsonSerializer.Deserialize<GetInformationResponse>(responseBody);

            OutputLog("[ユーザー情報取得]", url, httpResponse, response);
            return response;
        }

        private async Task<CreateTokenResponse> CreateToken()
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "token";
            HttpResponseMessage httpResponse = await client.PostAsync(url, null);
            string responseBody = await httpResponse.Content.ReadAsStringAsync();
            CreateTokenResponse response = JsonSerializer.Deserialize<CreateTokenResponse>(responseBody);

            OutputLog("[トークン作成]", url, httpResponse, response);
            return response;
        }

        private async Task<bool> AddTariff(string filePath)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "tariff";
            byte[] file = File.ReadAllBytes(filePath);
            ByteArrayContent content = new ByteArrayContent(file);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/octet-stream");
            HttpResponseMessage httpResponse = await client.PostAsync(url, content);
            string responseBody = await httpResponse.Content.ReadAsStringAsync();
            BaseResponse response = JsonSerializer.Deserialize<BaseResponse>(responseBody);

            OutputLog("[運賃表登録]", url, httpResponse, response);
            return response.ErrorCode == 0;
        }

        private async Task<bool> AddDistance(string filePath)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "distance";
            byte[] file = File.ReadAllBytes(filePath);
            ByteArrayContent content = new ByteArrayContent(file);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/octet-stream");
            HttpResponseMessage response = await client.PostAsync(url, content);
            string body = await response.Content.ReadAsStringAsync();
            BaseResponse baseResponse = JsonSerializer.Deserialize<BaseResponse>(body);

            OutputLog("[距離呈登録]", url, response, baseResponse);
            return baseResponse.ErrorCode == 0;
        }

        private async Task<bool> AddProfile(string filePath)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "profile";
            byte[] file = File.ReadAllBytes(filePath);
            ByteArrayContent content = new ByteArrayContent(file);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/octet-stream");
            HttpResponseMessage httpResponse = await client.PostAsync(url, content);
            string responseBody = await httpResponse.Content.ReadAsStringAsync();
            BaseResponse response = JsonSerializer.Deserialize<BaseResponse>(responseBody);

            OutputLog("[プロファイル登録]", url, httpResponse, response);
            return response.ErrorCode == 0;
        }

        private async Task<bool> CalcTsumiawase(int maxCalcRecord)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "profile/calc";
            CalcRequest request = new CalcRequest()
            {
                Id = "profile110",
                Date = "20211025",
                OutputDistances = true,
                Records = new List<object[]>(),
            };
            // 積合せ運賃計算レコードは取り纏めキー、重量、距離、発地住所コード、着地住所コードの順に入力してください
            request.Records.AddRange(new object[][] {
                new object[] { "", 10, 100, null,  null, },
                new object[] { "", 100, -1, "13103", "13102", },
                new object[] { "A", 10, 100, null, null, },
                new object[] { "A", 100, -1, "13103", "13102", },
            });
            return await CalcFare<CalcResponse>(url, request, maxCalcRecord);
        }

        private async Task<bool> CalcFare<T>(string url, CalcRequest request, int maxCalcRecord)
            where T : BaseResponse
        {
            if (maxCalcRecord <= 0)
            {
                return false;
            }

            // 1回でmaxCalcRecord数まで計算できます
            // maxCalcRecordを超える場合は分割して計算を繰り返してください
            var restRecords = request.Records;
            request.Records = new List<object[]>();
            while (restRecords.Count > 0)
            {
                request.Records.Clear();
                int size = Math.Min(restRecords.Count, maxCalcRecord);
                request.Records.AddRange(restRecords.GetRange(0, size));
                restRecords.RemoveRange(0, size);

                string requestBody = JsonSerializer.Serialize<CalcRequest>(request);
                var content = new StringContent(requestBody, Encoding.UTF8, "application/json");
                HttpResponseMessage httpResponse = await client.PostAsync(url, content);
                string responseBody = await httpResponse.Content.ReadAsStringAsync();
                T response = JsonSerializer.Deserialize<T>(responseBody);

                OutputLog("[運賃計算]", url, httpResponse, request, response);
                if (response.ErrorCode != 0)
                {
                    return false;
                }
            }
            return true;
        }

        private async Task<BaseResponse> DeleteToken()
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "token";
            HttpResponseMessage httpResponse = await client.DeleteAsync(url);
            string responseBody = await httpResponse.Content.ReadAsStringAsync();
            BaseResponse response = JsonSerializer.Deserialize<BaseResponse>(responseBody);

            OutputLog("[トークン削除]", url, httpResponse, response);
            return response;
        }

        private async Task<bool> CalcParcel(int maxCalcRecord)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "profile/calc";
            CalcRequest request = new CalcRequest()
            {
                Id = "profile140",
                Date = "20211025",
                Records = new List<object[]>(),
            };
            // 宅配運賃計算レコードは重量、サイズ、発地住所コード、着地住所コードの順に入力してください
            request.Records.AddRange(new object[][]
            {
                new object[] { 2, 60, "13103", "01000" },
                new object[] { 5, 60, "13104", "07000"},
                new object[] { 10, 60, "13201", "13000"},
                new object[] { 10, 140, "13202", "23000"},
                new object[] { 10, 160, "13103", "27000"},
                new object[] { 50, 170, "13103", "47000"},
            });
            return await CalcFare<SimpleCalcResponse>(url, request, maxCalcRecord);
        }

        private async Task<bool> CalcArea(int maxCalcRecord)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "profile/calc";
            CalcRequest request = new CalcRequest()
            {
                Id = "profile170",
                Date = "20211025",
                Records = new List<object[]>(),
            };
            // 地域別運賃計算レコードは取り纏めキー、重量、住所コードの順に入力してください
            request.Records.AddRange(new object[][]
            {
                new object[] { "", 1000, "13103", },
                new object[] { "", 2000, "13104", },
                new object[] { "", 3000, "13201", },
                new object[] { "", 4000, "13202", },
                new object[] { "A", 1000, "13103", },
                new object[] { "A", 9000, "13103", },
            });
            return await CalcFare<SimpleCalcResponse>(url, request, maxCalcRecord);
        }

        private async Task<bool> CalcCode(int maxCalcRecord)
        {
            string url = VirtualPathUtility.AppendTrailingSlash(textBoxUrl.Text) + "profile/calc";
            CalcRequest request = new CalcRequest()
            {
                Id = "profile180",
                Date = "20211025",
                Records = new List<object[]>(),
            };
            // 特殊コード運賃計算レコードは取り纏めキー、重量、特殊コードの順に入力してください
            request.Records.AddRange(new object[][]
            {
                new object[] { "", 5, "A", },
                new object[] { "", 6, "B", },
                new object[] { "", 4, "C", },
                new object[] { "", 2, "D", },
                new object[] { "A", 1, "A", },
                new object[] { "A", 9, "A", },
            });
            return await CalcFare<SimpleCalcResponse>(url, request, maxCalcRecord);
        }

        private void OutputLog<T>(string title, string url, HttpResponseMessage httpResponse,
            T responseJsonObj)
        {
            StringBuilder text = new StringBuilder();
            text.AppendLine(title);
            text.AppendLine($"URL = {url}");
            text.AppendLine($"Status Code = {(int)httpResponse.StatusCode}");
            JsonSerializerOptions jsonOptions = new JsonSerializerOptions()
            {
                Encoder = JavaScriptEncoder.Create(UnicodeRanges.All),
                WriteIndented = true,
            };
            text.AppendLine("Response Body = ");
            text.AppendLine(JsonSerializer.Serialize<T>(responseJsonObj, jsonOptions));
            text.AppendLine();
            textBoxLog.Text += text.ToString();
        }

        private void OutputLog<T, U>(string title, string url, HttpResponseMessage httpResponse,
            T requestJsonObj, U responseJsonObj)
        {
            StringBuilder text = new StringBuilder();
            text.AppendLine(title);
            text.AppendLine($"URL = {url}");
            text.AppendLine($"Status Code = {(int)httpResponse.StatusCode}");
            JsonSerializerOptions jsonOptions = new JsonSerializerOptions()
            {
                Encoder = JavaScriptEncoder.Create(UnicodeRanges.All),
                WriteIndented = true,
            };
            if (requestJsonObj != null)
            {
                text.AppendLine("Request Body = ");
                text.AppendLine(JsonSerializer.Serialize<T>(requestJsonObj, jsonOptions));
            }
            if (responseJsonObj != null)
            {
                text.AppendLine("Response Body = ");
                text.AppendLine(JsonSerializer.Serialize<U>(responseJsonObj, jsonOptions));
            }
            text.AppendLine();
            textBoxLog.Text += text.ToString();
        }
    }
}
